@@ -71,7 +71,7 @@
  * See the README file for information on usage and redistribution.
  */
 
-#define PILLOW_VERSION "2.5.2"
+#define PILLOW_VERSION "3.2.0"
 
 #include "Python.h"
 
@@ -98,9 +98,6 @@
 
 #define WITH_DEBUG /* extra debugging interfaces */
 
-/* PIL Plus extensions */
-#undef  WITH_CRACKCODE /* pil plus */
-
 #undef    VERBOSE
 
 #define CLIP(x) ((x) <= 0 ? 0 : (x) < 256 ? (x) : 255)
@@ -365,8 +362,11 @@ getbands(const char* mode)
 static void*
 getlist(PyObject* arg, int* length, const char* wrong_length, int type)
 {
-    int i, n;
+    int i, n, itemp;
+    double dtemp;
     void* list;
+    PyObject* seq;
+    PyObject* op;
 
     if (!PySequence_Check(arg)) {
         PyErr_SetString(PyExc_TypeError, must_be_sequence);
@@ -383,77 +383,42 @@ getlist(PyObject* arg, int* length, const char* wrong_length, int type)
     if (!list)
         return PyErr_NoMemory();
 
-    switch (type) {
-    case TYPE_UINT8:
-        if (PyList_Check(arg)) {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PyList_GET_ITEM(arg, i);
-                int temp = PyInt_AsLong(op);
-                ((UINT8*)list)[i] = CLIP(temp);
-            }
-        } else {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PySequence_GetItem(arg, i);
-                int temp = PyInt_AsLong(op);
-                Py_XDECREF(op);
-                ((UINT8*)list)[i] = CLIP(temp);
-            }
+    seq = PySequence_Fast(arg, must_be_sequence);
+    if (!seq) {
+        free(list);
+        PyErr_SetString(PyExc_TypeError, must_be_sequence);
+        return NULL;
+    }
+
+    for (i = 0; i < n; i++) {
+        op = PySequence_Fast_GET_ITEM(seq, i);
+        // DRY, branch prediction is going to work _really_ well
+        // on this switch. And 3 fewer loops to copy/paste.
+        switch (type) {
+        case TYPE_UINT8:
+            itemp = PyInt_AsLong(op);
+            ((UINT8*)list)[i] = CLIP(itemp);
+            break;
+        case TYPE_INT32:
+            itemp = PyInt_AsLong(op);
+            ((INT32*)list)[i] = itemp;
+            break;
+        case TYPE_FLOAT32:
+            dtemp = PyFloat_AsDouble(op);
+            ((FLOAT32*)list)[i] = (FLOAT32) dtemp;
+            break;
+        case TYPE_DOUBLE:
+            dtemp = PyFloat_AsDouble(op);
+            ((double*)list)[i] = (double) dtemp;
+            break;
         }
-        break;
-    case TYPE_INT32:
-        if (PyList_Check(arg)) {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PyList_GET_ITEM(arg, i);
-                int temp = PyInt_AsLong(op);
-                ((INT32*)list)[i] = temp;
-            }
-        } else {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PySequence_GetItem(arg, i);
-                int temp = PyInt_AsLong(op);
-                Py_XDECREF(op);
-                ((INT32*)list)[i] = temp;
-            }
-        }
-        break;
-    case TYPE_FLOAT32:
-        if (PyList_Check(arg)) {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PyList_GET_ITEM(arg, i);
-                double temp = PyFloat_AsDouble(op);
-                ((FLOAT32*)list)[i] = (FLOAT32) temp;
-            }
-        } else {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PySequence_GetItem(arg, i);
-                double temp = PyFloat_AsDouble(op);
-                Py_XDECREF(op);
-                ((FLOAT32*)list)[i] = (FLOAT32) temp;
-            }
-        }
-        break;
-    case TYPE_DOUBLE:
-        if (PyList_Check(arg)) {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PyList_GET_ITEM(arg, i);
-                double temp = PyFloat_AsDouble(op);
-                ((double*)list)[i] = temp;
-            }
-        } else {
-            for (i = 0; i < n; i++) {
-                PyObject *op = PySequence_GetItem(arg, i);
-                double temp = PyFloat_AsDouble(op);
-                Py_XDECREF(op);
-                ((double*)list)[i] = temp;
-            }
-        }
-        break;
     }
 
     if (length)
         *length = n;
 
     PyErr_Clear();
+    Py_DECREF(seq);
 
     return list;
 }
@@ -506,35 +471,53 @@ getpixel(Imaging im, ImagingAccess access, int x, int y)
 static char*
 getink(PyObject* color, Imaging im, char* ink)
 {
-    int r, g, b, a;
-    double f;
+    int g=0, b=0, a=0;
+    double f=0;
+    /* Windows 64 bit longs are 32 bits, and 0xFFFFFFFF (white) is a
+       python long (not int) that raises an overflow error when trying
+       to return it into a 32 bit C long
+    */
+    PY_LONG_LONG r = 0;
 
     /* fill ink buffer (four bytes) with something that can
        be cast to either UINT8 or INT32 */
 
+    int rIsInt = 0;
+    if (im->type == IMAGING_TYPE_UINT8 ||
+        im->type == IMAGING_TYPE_INT32 ||
+        im->type == IMAGING_TYPE_SPECIAL) {
+#if PY_VERSION_HEX >= 0x03000000
+                if (PyLong_Check(color)) {
+                        r = PyLong_AsLongLong(color);
+#else
+                if (PyInt_Check(color) || PyLong_Check(color)) {
+                        if (PyInt_Check(color))
+                                r = PyInt_AS_LONG(color);
+                        else
+                                r = PyLong_AsLongLong(color);
+#endif
+            rIsInt = 1;
+                }
+                if (r == -1 && PyErr_Occurred()) {
+                    rIsInt = 0;
+        }
+    }
+
     switch (im->type) {
     case IMAGING_TYPE_UINT8:
         /* unsigned integer */
         if (im->bands == 1) {
             /* unsigned integer, single layer */
-            r = PyInt_AsLong(color);
-            if (r == -1 && PyErr_Occurred())
-                return NULL;
+            if (rIsInt != 1) {
+                if (!PyArg_ParseTuple(color, "i", &r)) {
+                    return NULL;
+                }
+            }
             ink[0] = CLIP(r);
             ink[1] = ink[2] = ink[3] = 0;
         } else {
             a = 255;
-#if PY_VERSION_HEX >= 0x03000000
-            if (PyLong_Check(color)) {
-                r = (int) PyLong_AsLong(color);
-#else
-            if (PyInt_Check(color) || PyLong_Check(color)) {
-                if (PyInt_Check(color))
-                    r = PyInt_AS_LONG(color);
-                else
-                    r = (int) PyLong_AsLong(color);
-#endif
-
+            if (rIsInt) {
                 /* compatibility: ABGR */
                 a = (UINT8) (r >> 24);
                 b = (UINT8) (r >> 16);
@@ -558,8 +541,7 @@ getink(PyObject* color, Imaging im, char* ink)
         return ink;
     case IMAGING_TYPE_INT32:
         /* signed integer */
-        r = PyInt_AsLong(color);
-        if (r == -1 && PyErr_Occurred())
+        if (rIsInt != 1)
             return NULL;
         *(INT32*) ink = r;
         return ink;
@@ -572,8 +554,7 @@ getink(PyObject* color, Imaging im, char* ink)
         return ink;
     case IMAGING_TYPE_SPECIAL:
         if (strncmp(im->mode, "I;16", 4) == 0) {
-            r = PyInt_AsLong(color);
-            if (r == -1 && PyErr_Occurred())
+            if (rIsInt != 1)
                 return NULL;
             ink[0] = (UINT8) r;
             ink[1] = (UINT8) (r >> 8);
@@ -609,13 +590,14 @@ _fill(PyObject* self, PyObject* args)
     if (!im)
         return NULL;
 
+    buffer[0] = buffer[1] = buffer[2] = buffer[3] = 0;
     if (color) {
         if (!getink(color, im, buffer)) {
             ImagingDelete(im);
             return NULL;
         }
-    } else
-        buffer[0] = buffer[1] = buffer[2] = buffer[3] = 0;
+    }
+
 
     (void) ImagingFill(im, buffer);
 
@@ -784,11 +766,12 @@ _convert_matrix(ImagingObject* self, PyObject* args)
     float m[12];
     if (!PyArg_ParseTuple(args, "s(ffff)", &mode, m+0, m+1, m+2, m+3)) {
         PyErr_Clear();
-    if (!PyArg_ParseTuple(args, "s(ffffffffffff)", &mode,
-                  m+0, m+1, m+2, m+3,
-                  m+4, m+5, m+6, m+7,
-                  m+8, m+9, m+10, m+11))
-        return NULL;
+        if (!PyArg_ParseTuple(args, "s(ffffffffffff)", &mode,
+                              m+0, m+1, m+2, m+3,
+                              m+4, m+5, m+6, m+7,
+                              m+8, m+9, m+10, m+11)){
+            return NULL;
+        }
     }
 
     return PyImagingNew(ImagingConvertMatrix(self->image, mode, m));
@@ -896,7 +879,8 @@ _gaussian_blur(ImagingObject* self, PyObject* args)
     Imaging imOut;
 
     float radius = 0;
-    if (!PyArg_ParseTuple(args, "f", &radius))
+    int passes = 3;
+    if (!PyArg_ParseTuple(args, "f|i", &radius, &passes))
         return NULL;
 
     imIn = self->image;
@@ -904,7 +888,7 @@ _gaussian_blur(ImagingObject* self, PyObject* args)
     if (!imOut)
         return NULL;
 
-    if (!ImagingGaussianBlur(imIn, imOut, radius))
+    if (!ImagingGaussianBlur(imOut, imIn, radius, passes))
         return NULL;
 
     return PyImagingNew(imOut);
@@ -1253,6 +1237,8 @@ _putdata(ImagingObject* self, PyObject* args)
     Py_ssize_t n, i, x, y;
 
     PyObject* data;
+    PyObject* seq = NULL;
+    PyObject* op;
     double scale = 1.0;
     double offset = 0.0;
 
@@ -1292,69 +1278,61 @@ _putdata(ImagingObject* self, PyObject* args)
                         x = 0, y++;
                 }
         } else {
-            if (scale == 1.0 && offset == 0.0) {
-                /* Clipped data */
-                if (PyList_Check(data)) {
-                    for (i = x = y = 0; i < n; i++) {
-                        PyObject *op = PyList_GET_ITEM(data, i);
-                        image->image8[y][x] = (UINT8) CLIP(PyInt_AsLong(op));
-                        if (++x >= (int) image->xsize)
-                            x = 0, y++;
-                    }
-                } else {
-                    for (i = x = y = 0; i < n; i++) {
-                        PyObject *op = PySequence_GetItem(data, i);
-                        image->image8[y][x] = (UINT8) CLIP(PyInt_AsLong(op));
-                        Py_XDECREF(op);
-                        if (++x >= (int) image->xsize)
-                            x = 0, y++;
-                    }
-                }
+           seq = PySequence_Fast(data, must_be_sequence);
+           if (!seq) {
+               PyErr_SetString(PyExc_TypeError, must_be_sequence);
+               return NULL;
+           }
+           if (scale == 1.0 && offset == 0.0) {
+               /* Clipped data */
+               for (i = x = y = 0; i < n; i++) {
+                   op = PySequence_Fast_GET_ITEM(seq, i);
+                   image->image8[y][x] = (UINT8) CLIP(PyInt_AsLong(op));
+                   if (++x >= (int) image->xsize){
+                       x = 0, y++;
+                   }
+               }
+
             } else {
-                if (PyList_Check(data)) {
-                    /* Scaled and clipped data */
-                    for (i = x = y = 0; i < n; i++) {
-                        PyObject *op = PyList_GET_ITEM(data, i);
-                        image->image8[y][x] = CLIP(
-                            (int) (PyFloat_AsDouble(op) * scale + offset));
-                        if (++x >= (int) image->xsize)
-                            x = 0, y++;
-                    }
-                } else {
-                    for (i = x = y = 0; i < n; i++) {
-                        PyObject *op = PySequence_GetItem(data, i);
-                        image->image8[y][x] = CLIP(
-                            (int) (PyFloat_AsDouble(op) * scale + offset));
-                        Py_XDECREF(op);
-                        if (++x >= (int) image->xsize)
-                            x = 0, y++;
-                    }
-                }
-            }
-            PyErr_Clear(); /* Avoid weird exceptions */
+               /* Scaled and clipped data */
+               for (i = x = y = 0; i < n; i++) {
+                   PyObject *op = PySequence_Fast_GET_ITEM(seq, i);
+                   image->image8[y][x] = CLIP(
+                       (int) (PyFloat_AsDouble(op) * scale + offset));
+                   if (++x >= (int) image->xsize){
+                       x = 0, y++;
+                   }
+               }
+           }
+           PyErr_Clear(); /* Avoid weird exceptions */
         }
     } else {
         /* 32-bit images */
+        seq = PySequence_Fast(data, must_be_sequence);
+        if (!seq) {
+            PyErr_SetString(PyExc_TypeError, must_be_sequence);
+            return NULL;
+        }
         switch (image->type) {
         case IMAGING_TYPE_INT32:
             for (i = x = y = 0; i < n; i++) {
-                PyObject *op = PySequence_GetItem(data, i);
+                op = PySequence_Fast_GET_ITEM(seq, i);
                 IMAGING_PIXEL_INT32(image, x, y) =
                     (INT32) (PyFloat_AsDouble(op) * scale + offset);
-                Py_XDECREF(op);
-                if (++x >= (int) image->xsize)
+                if (++x >= (int) image->xsize){
                     x = 0, y++;
+                }
             }
             PyErr_Clear(); /* Avoid weird exceptions */
             break;
         case IMAGING_TYPE_FLOAT32:
             for (i = x = y = 0; i < n; i++) {
-                PyObject *op = PySequence_GetItem(data, i);
+                op = PySequence_Fast_GET_ITEM(seq, i);
                 IMAGING_PIXEL_FLOAT32(image, x, y) =
                     (FLOAT32) (PyFloat_AsDouble(op) * scale + offset);
-                Py_XDECREF(op);
-                if (++x >= (int) image->xsize)
+                if (++x >= (int) image->xsize){
                     x = 0, y++;
+                }
             }
             PyErr_Clear(); /* Avoid weird exceptions */
             break;
@@ -1365,22 +1343,26 @@ _putdata(ImagingObject* self, PyObject* args)
                     INT32 inkint;
                 } u;
 
-                PyObject *op = PySequence_GetItem(data, i);
+                u.inkint = 0;
+
+                op = PySequence_Fast_GET_ITEM(seq, i);
                 if (!op || !getink(op, image, u.ink)) {
-                    Py_DECREF(op);
+                    Py_DECREF(seq);
                     return NULL;
                 }
                 /* FIXME: what about scale and offset? */
                 image->image32[y][x] = u.inkint;
-                Py_XDECREF(op);
-                if (++x >= (int) image->xsize)
+                if (++x >= (int) image->xsize){
                     x = 0, y++;
+                }
             }
             PyErr_Clear(); /* Avoid weird exceptions */
             break;
         }
     }
 
+    Py_XDECREF(seq);
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -1553,9 +1535,30 @@ _resize(ImagingObject* self, PyObject* args)
 
     imIn = self->image;
 
-    imOut = ImagingNew(imIn->mode, xsize, ysize);
-    if (imOut)
-        (void) ImagingResize(imOut, imIn, filter);
+    if (xsize < 1 || ysize < 1) {
+        return ImagingError_ValueError("height and width must be > 0");
+    }
+
+    if (imIn->xsize == xsize && imIn->ysize == ysize) {
+        imOut = ImagingCopy(imIn);
+    }
+    else if ( ! filter) {
+        double a[6];
+
+        memset(a, 0, sizeof a);
+        a[1] = (double) imIn->xsize / xsize;
+        a[5] = (double) imIn->ysize / ysize;
+
+        imOut = ImagingNew(imIn->mode, xsize, ysize);
+
+        imOut = ImagingTransformAffine(
+            imOut, imIn,
+            0, 0, xsize, ysize,
+            a, filter, 1);
+    }
+    else {
+        imOut = ImagingResample(imIn, xsize, ysize, filter);
+    }
 
     return PyImagingNew(imOut);
 }
@@ -1568,7 +1571,8 @@ _rotate(ImagingObject* self, PyObject* args)
 
     double theta;
     int filter = IMAGING_TRANSFORM_NEAREST;
-    if (!PyArg_ParseTuple(args, "d|i", &theta, &filter))
+    int expand;
+    if (!PyArg_ParseTuple(args, "d|i|i", &theta, &filter, &expand))
         return NULL;
 
     imIn = self->image;
@@ -1581,7 +1585,8 @@ _rotate(ImagingObject* self, PyObject* args)
         /* Rotate with resampling filter */
         imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
     (void) ImagingRotate(imOut, imIn, theta, filter);
-    } else if (theta == 90.0 || theta == 270.0) {
+    } else if ((theta == 90.0 || theta == 270.0)
+            && (expand || imIn->xsize == imIn->ysize)) {
         /* Use fast version */
         imOut = ImagingNew(imIn->mode, imIn->ysize, imIn->xsize);
         if (imOut) {
@@ -1649,51 +1654,6 @@ im_setmode(ImagingObject* self, PyObject* args)
     return Py_None;
 }
 
-static PyObject*
-_stretch(ImagingObject* self, PyObject* args)
-{
-    Imaging imIn;
-    Imaging imTemp;
-    Imaging imOut;
-
-    int xsize, ysize;
-    int filter = IMAGING_TRANSFORM_NEAREST;
-    if (!PyArg_ParseTuple(args, "(ii)|i", &xsize, &ysize, &filter))
-    return NULL;
-
-    imIn = self->image;
-
-    /* two-pass resize: minimize size of intermediate image */
-    if ((Py_ssize_t) imIn->xsize * ysize < (Py_ssize_t) xsize * imIn->ysize)
-        imTemp = ImagingNew(imIn->mode, imIn->xsize, ysize);
-    else
-        imTemp = ImagingNew(imIn->mode, xsize, imIn->ysize);
-    if (!imTemp)
-        return NULL;
-
-    /* first pass */
-    if (!ImagingStretch(imTemp, imIn, filter)) {
-        ImagingDelete(imTemp);
-        return NULL;
-    }
-
-    imOut = ImagingNew(imIn->mode, xsize, ysize);
-    if (!imOut) {
-        ImagingDelete(imTemp);
-        return NULL;
-    }
-
-    /* second pass */
-    if (!ImagingStretch(imOut, imTemp, filter)) {
-        ImagingDelete(imOut);
-        ImagingDelete(imTemp);
-        return NULL;
-    }
-
-    ImagingDelete(imTemp);
-
-    return PyImagingNew(imOut);
-}
 
 static PyObject*
 _transform2(ImagingObject* self, PyObject* args)
@@ -1790,6 +1750,7 @@ _transpose(ImagingObject* self, PyObject* args)
         break;
     case 2: /* rotate 90 */
     case 4: /* rotate 270 */
+    case 5: /* transpose */
         imOut = ImagingNew(imIn->mode, imIn->ysize, imIn->xsize);
         break;
     default:
@@ -1814,6 +1775,9 @@ _transpose(ImagingObject* self, PyObject* args)
         case 4:
             (void) ImagingRotate270(imOut, imIn);
             break;
+        case 5:
+            (void) ImagingTranspose(imOut, imIn);
+            break;
         }
 
     return PyImagingNew(imOut);
@@ -1831,18 +1795,39 @@ _unsharp_mask(ImagingObject* self, PyObject* args)
     if (!PyArg_ParseTuple(args, "fii", &radius, &percent, &threshold))
         return NULL;
 
+    imIn = self->image;
+    imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
+    if (!imOut)
+        return NULL;
+
+    if (!ImagingUnsharpMask(imOut, imIn, radius, percent, threshold))
+        return NULL;
+
+    return PyImagingNew(imOut);
+}
+#endif
+
+static PyObject*
+_box_blur(ImagingObject* self, PyObject* args)
+{
+    Imaging imIn;
+    Imaging imOut;
+
+    float radius;
+    int n = 1;
+    if (!PyArg_ParseTuple(args, "f|i", &radius, &n))
+        return NULL;
 
     imIn = self->image;
     imOut = ImagingNew(imIn->mode, imIn->xsize, imIn->ysize);
     if (!imOut)
         return NULL;
 
-    if (!ImagingUnsharpMask(imIn, imOut, radius, percent, threshold))
+    if (!ImagingBoxBlur(imOut, imIn, radius, n))
         return NULL;
 
     return PyImagingNew(imOut);
 }
-#endif
 
 /* -------------------------------------------------------------------- */
 
@@ -2404,9 +2389,9 @@ _draw_arc(ImagingDrawObject* self, PyObject* args)
 
     PyObject* data;
     int ink;
-    int start, end;
+    float start, end;
     int op = 0;
-    if (!PyArg_ParseTuple(args, "Oiii|i", &data, &start, &end, &ink))
+    if (!PyArg_ParseTuple(args, "Offi|i", &data, &start, &end, &ink))
         return NULL;
 
     n = PyPath_Flatten(data, &xy);
@@ -2478,8 +2463,8 @@ _draw_chord(ImagingDrawObject* self, PyObject* args)
 
     PyObject* data;
     int ink, fill;
-    int start, end;
-    if (!PyArg_ParseTuple(args, "Oiiii",
+    float start, end;
+    if (!PyArg_ParseTuple(args, "Offii",
                           &data, &start, &end, &ink, &fill))
         return NULL;
 
@@ -2699,8 +2684,8 @@ _draw_pieslice(ImagingDrawObject* self, PyObject* args)
 
     PyObject* data;
     int ink, fill;
-    int start, end;
-    if (!PyArg_ParseTuple(args, "Oiiii", &data, &start, &end, &ink, &fill))
+    float start, end;
+    if (!PyArg_ParseTuple(args, "Offii", &data, &start, &end, &ink, &fill))
         return NULL;
 
     n = PyPath_Flatten(data, &xy);
@@ -3049,9 +3034,6 @@ static struct PyMethodDef methods[] = {
     {"convert_transparent", (PyCFunction)_convert_transparent, 1},
     {"copy", (PyCFunction)_copy, 1},
     {"copy2", (PyCFunction)_copy2, 1},
-#ifdef WITH_CRACKCODE
-    {"crackcode", (PyCFunction)_crackcode, 1},
-#endif
     {"crop", (PyCFunction)_crop, 1},
     {"expand", (PyCFunction)_expand_image, 1},
     {"filter", (PyCFunction)_filter, 1},
@@ -3071,8 +3053,10 @@ static struct PyMethodDef methods[] = {
     {"rankfilter", (PyCFunction)_rankfilter, 1},
 #endif
     {"resize", (PyCFunction)_resize, 1},
+    // There were two methods for image resize before.
+    // Starting from Pillow 2.7.0 stretch is depreciated.
+    {"stretch", (PyCFunction)_resize, 1},
     {"rotate", (PyCFunction)_rotate, 1},
-    {"stretch", (PyCFunction)_stretch, 1},
     {"transpose", (PyCFunction)_transpose, 1},
     {"transform2", (PyCFunction)_transform2, 1},
 
@@ -3118,6 +3102,8 @@ static struct PyMethodDef methods[] = {
     {"unsharp_mask", (PyCFunction)_unsharp_mask, 1},
 #endif
 
+    {"box_blur", (PyCFunction)_box_blur, 1},
+
 #ifdef WITH_EFFECTS
     /* Special effects */
     {"effect_spread", (PyCFunction)_effect_spread, 1},
